/*

Xiangqi Game

(C) Andrei Barbu 2003-2004

This software is distributed as is, under all the terms of the GPL.

*/


#include "main.h"


#ifdef WIN32 
#       include <windows.h>
#       include <crtdbg.h>
#       ifdef _DEBUG
#               pragma comment( lib , "sdld.lib" )
#               pragma comment( lib , "sdl_imaged.lib" )
#               pragma comment( lib , "sdl_ttfd.lib" )
#       else
#               pragma comment( lib , "sdl.lib" )
#               pragma comment( lib , "sdl_image.lib" )
#               pragma comment( lib , "sdl_ttf.lib" )
#       endif
#endif

// ---- Let's see about those globals
Player* player[2];
Turn turn; 	// The current turn, 1 for black, 0 for red, red should start
		// 3 is an error, or no game

SDL_Surface *board;	// The surface of the board and screen
SDL_Surface *screen;	// Like this we don't have to pass them around constantly
// ----

int main()
{
    screen = SI_Setup( 1024 , 768 ); //Let's set SDL up

    if( screen == NULL )
	return 1;
    atexit( SDL_Quit );	
    
//    if( TI_Setup() ) //Now we can see if SDL_ttf works
//	return 1;

    if( Game_Main( screen ) != 0 )
        printf( "Game_Main exited with an error" );
    
    return 0;
}

int Game_Main( SDL_Surface* screen )
{    
    player[0] = new Player( 0 ); // Red
    player[1] = new Player( 1 ); // Black
    
    if( player[0] == NULL || player[1] == NULL )
        return 1;
    
    board = SI_DrawBoard( 250 , 70 , screen );

    SDL_UpdateRect( screen , 0 , 0 , 0 , 0);

    if( player[0]->load_pieces() != 0 || player[1]->load_pieces() != 0)
	printf( "Error loading pieces %s\n", IMG_GetError() );
	
	
    player[0]->show_pieces( screen );
    player[1]->show_pieces( screen );

    Game_Loop( screen, board , player ); // Main game loop
    
    if( player[0] )    
	delete player[0];
    if( player[1] )
	delete player[1];

    return 0;
}

int Game_Loop( SDL_Surface* screen , SDL_Surface* board , Player** player )
{

    SDL_Event event;
    int xmove = 0;
    int ymove = 0;

    turn.start(); // 0 = Red, 1 = Black, 2 on error, or no game
	    // Red starts
		  
    while( SDL_WaitEvent(&event) != 0 )
    {
	SDL_keysym keysym;
	SDL_ActiveEvent active;
	piece tmp_piece;

	switch( event.type )
	{
	    case SDL_MOUSEBUTTONDOWN:
	    
		PDEBUG("Mouse pressed (%i) at (%i,%i), position (%i,%i)\n" , event.button.button ,			// Print out mouse positions and clicks
		    event.button.x , event.button.y , SI_Row( event.button.x ) , SI_Col( event.button.y) );
		    PDEBUG("Piece? %i\n",  player[turn.curr()]->is_piece( SI_Row(event.button.x) , SI_Col(event.button.y)) );

		if( SI_Row(event.button.x) != 0 && SI_Col(event.button.y) != 0 )	// If we are on the board
		    if( (xmove == 0 || ymove == 0) && player[turn.curr()]->is_piece( SI_Row( event.button.x ) , SI_Col( event.button.y ) ) )
			{
			    xmove = SI_Row(event.button.x);
			    ymove = SI_Col(event.button.y);
			    PDEBUG("piece chosen, %i, %i\n", xmove, ymove );
			}
			else
			{
			    PDEBUG("time to move: %i, %i, %i, %i \n", xmove, ymove, SI_Row( event.button.x ) , SI_Col( event.button.y ));

			    // If the move is valid, do it and we'll clean up
			    if( ( player[turn.curr()]->move( xmove , ymove ,  SI_Row(event.button.x) , SI_Col(event.button.y) , turn ) ) )
			    {
				// Eventually make the GUI_Redraw function handle all this
				SI_ReDrawBoard( board , 250 , 70 , screen );
				player[0]->show_pieces( screen );
				player[1]->show_pieces( screen );

				++turn;
			    }
			    
			    xmove = 0;
			    ymove = 0;

			    
			}
			
		SDL_UpdateRect( screen , 0 , 0 , 0 , 0 );
		break;
		
	    case SDL_QUIT:
		printf("Quit detected\n");
		return 0;
	    break;
		
	    case SDL_KEYDOWN:

		keysym = event.key.keysym;
		if( keysym.sym == SDLK_q )
		{
		    PDEBUG("Q pressed, exiting now\n"); 
		    return 0;
		}

		if( keysym.sym == SDLK_n )
		{
		    PDEBUG("N pressed, reseting players\n");

		    if( Game_New() )
			return 1;

		}
	    
	    break;

	    case SDL_APPMOUSEFOCUS:

		if( active.gain == 1 )
		{
		    SDL_UpdateRect( screen , 0 , 0 , 0 , 0 );
		}
		
	    break;
	}
    }	
    
    return 0;
}

int Game_New()
{

    if( player[0] )		// Clear the players
	delete player[0];
    if( player[1] )
	delete player[1];
	    
    player[0] = new Player( 0 ); // Set up a new set of plaers
    player[1] = new Player( 1 );
	    
    if( player[0]->load_pieces() != 0 || player[1]->load_pieces() != 0 )	// Give them pieces
    {
	printf( "Error loading pieces %s\n", IMG_GetError() );
	return 1;
    }
    
    turn.start();					// Reset the turn counter
	    
    SI_ReDrawBoard( board , 250 , 70 , screen );	// Redraw the board and set up the pieces
    player[0]->show_pieces( screen );
    player[1]->show_pieces( screen );
    SDL_UpdateRect( screen , 0 , 0 ,0 , 0 );
    
    return 0;
}


