/*

Xiangqi Game

(C) Andrei Barbu 2003-2004

This software is distributed as is, under all the terms of the GPL.

*/

#include "piece.h"

// ---- Globals

extern Player* player[3]; //Let's get both players

// ---- 


int piece::p_valid_move( int dx, int dy , Turn turn )
{

    if( !turn.valid() ) // Error if no game
	return 0;
	
    if( player[turn.curr()]->is_piece( dx , dy ) ) 	// If the owner of this piece, has a piece were this one is heading, fail
	return 0;				// Doesn't matter what type of piece

    switch( type )
    {
	case CHARIOT:

	    if( x != dx && y != dy ) //Only horizonatally
		return 0;

	    // Check if there is a piece between us, and the square we want to get to
	    if( dx > x )
		for( int tx = x + 1; tx < dx; tx++ )
		    if( is_piece( tx , y ) )
			return 0;
	    if( dx < x )
		for( int tx = x - 1; tx > dx; tx-- )
		    if( is_piece( tx , y) )
			return 0;

	    if( y < dy )
    		for( int ty = y + 1; ty < dy; ty++ )
		    if( is_piece( x , ty ) )
			return 0;
	    if( y > dy )
		for( int ty = y - 1; ty > dy; ty-- )
		    if( is_piece( x , ty ) )
			return 0;

	break;

	case CANNON:
	    if( x != dx && y != dy ) //Only horizontally
		return 0;
	    if( !is_piece( dx , dy ) ) //If there is no piece were we are heading, and something in bewtween, fail, miserably
	    {
		if( dx > x )
		    for( int tx = x + 1; tx < dx; tx++ )
			if( is_piece( tx , y ) )
			    return 0;
		if( dx < x )
		    for( int tx = x - 1; tx > dx; tx-- )
			if( is_piece( tx , y) )
			    return 0;

		if( y < dy )
    		    for( int ty = y + 1; ty < dy; ty++ )
			if( is_piece( x , ty ) )
			    return 0;
		if( y > dy )
		    for( int ty = y - 1; ty > dy; ty-- )
			if( is_piece( x , ty ) )
			    return 0;
	    }
	    
	    // We can only take a piece if we jump over 1 piece
	    if( player[++turn]->is_piece( dx , dy ) )
	    {
		++turn;
		int jump_pieces = 0;
		if( dx > x )
		    for( int tx = x + 1; tx < dx; tx++ )
			if( is_piece( tx , y ) )
			    jump_pieces++;
		if( dx < x )
		    for( int tx = x - 1; tx > dx; tx-- )
			if( is_piece( tx , y) )
			    jump_pieces++;

		if( y < dy )
    		    for( int ty = y + 1; ty < dy; ty++ )
			if( is_piece( x , ty ) )
			    jump_pieces++;
		if( y > dy )
		    for( int ty = y - 1; ty > dy; ty-- )
			if( is_piece( x , ty ) )
			    jump_pieces++;	    
		if( jump_pieces != 1 )
		    return 0;
	    }

	break;

	case ELEPHANT:
	    if( (abs( dx - x ) != 2) || (abs( dy - y) != 2) ) //Only move 2 diagonally
		return 0;
	    if( (y < 6 && dy > 5) || (y > 5 && dy < 6) ) //Stop at the river
		return 0;
	    if( dx > x && dy > y )
		if( is_piece( x + 1 , y + 1 ) )
		    return 0;
	    if( dx > x && dy < y )
		if( is_piece( x + 1 , y - 1 ) )
		    return 0;
	    if( dx < x && dy > y )
		if( is_piece( x - 1 , y + 1 ) )
		    return 0;
	    if( dx < x && dy < y )
		if( is_piece( x - 1 , y - 1 ) ) 
		    return 0;
	break;

	case KNIGHT:
	    if( !( ((abs( dx - x ) == 1) && (abs( dy - y ) == 2)) || ((abs( dx - x ) == 2) && (abs( dy - y) == 1)) ) ) 	//One in the direction, and one across
	    	return 0;												// -\ shape


	    // Since the first move doesn't jump over anything, we have to check if we are being blocked
	    if( abs( dx - x ) == 2 )
	    {
	        if( dx > x )
		    if( is_piece( x + 1 , y ) )
			return 0;
		if( dx < x )
		    if( is_piece( x - 1 , y ) )
			return 0;
	    }
	    else
	    {
		if( dy > y )
		    if( is_piece( x , y + 1 ) )
			return 0;
		if( dy < y )
		    if( is_piece( x , y - 1 ) ) 
			return 0;	    
	    }
	    
	break;

	case KING:

//TODO: The fact that the two kings can't face eachother
	    
	    if( dx < 4 || dx > 6 )	// Stay inside the castle
		return 0;
	    if( (y < 4 && dy > 3) || (y > 7 && dy < 8 ) )  // ditto
		return 0;
	    if( x != dx  && y != dy ) // Only horizontally
		return 0;
	    if( ( abs( x - dx ) != 1) && (abs( y - dy ) != 1) ) //And only 1 space
		    return 0;

	break;

	case GUARD:
	    if( dx < 4 || dx > 6 ) // Inside the castle
		return 0;
	    if( (y < 4 && dy > 3) || (y > 7 && dy < 8 ) ) // Stay there
		return 0;
	    if( x == dx || y == dy ) // Only diagonally
		return 0;
	    if( (abs( x - dx ) != 1) && (abs( y - dy ) != 1) ) //And only 1 space
		return 0;
	    if( player[turn.curr()]->is_piece( dx , dy ) ) //Fail if the player has a piece were this one is going
		return 0;
	break;

	case PAWN:
	    if( x != dx && y != dy ) //Only horizontal
		return 0;
	    if( (y < dy && y < 6 && flag[2] == 1) || (y > dy && y > 5 && flag[2] == 1) ) //After the river, make sure we don't let it go back
		return 0;
	    if( x != dx && flag[2] == 0 )						//Fail to move horizontally if we haven't passed the river
	    	return 0;
	    if( (y < 6 && dy < y && flag[2] == 0) || (y > 5 && dy > y && flag[2] == 0) ) //Before the river make sure it doesn't go back
	    	return 0;
	    if( (abs( x - dx ) != 1) &&  (abs( y - dy) != 1) ) //Make sure it only moves 1 spot
		return 0;
	break;
		    
	default: return 0; // Technically this isn't required
    }

    return 1;

}

int is_piece( int row , int col )  // Is there a piece at x , y
{
    for( int a = 0; a < 2; a++ )
	for( unsigned i = 0; i < player[a]->pieces.size(); i++ )
	    if( player[a]->pieces[i]->x == row && player[a]->pieces[i]->y == col )
		return 1;
    return 0;
}

void piece::p_print()
{
    PDEBUG("%i, %i, %i, %i\n", x, y, flag[2], type);
}

bool piece::pawn_promote( int dx , int dy ) // If it's a pawn and it crosses 5 & 6 promote it, true on promote
{
    if( type == PAWN)
        if( ( (y < 6 && dy > 5) || (y > 5 && dy < 6) ) && flag[2] == 0 )   // Promote the pawn if it passes
	{
    	    flag[2] = 1;                                                                         // the river, between 5 & 6
	    return true;
	}
    return false;    

}

